// Student: Avram Isabela-Maria
// Grupa: 331CA

// Libraries used
#include <Wire.h>
#include <Keypad.h>
#include <LiquidCrystal_I2C.h>
#include <Servo.h>
#include <pitches.h>  // Library for the notes of the songs

//=============================SONGS==================================

// The buzzer which plays the songs
#define BUZZER 9

// Lion sleeps tonight:
int melodyL[] = {
  NOTE_F4, NOTE_G4, NOTE_A4, NOTE_G4, NOTE_A4,
  NOTE_AS4, NOTE_A4, NOTE_G4, NOTE_F4, NOTE_G4,
  NOTE_A4, NOTE_C4, NOTE_C4, NOTE_C4, NOTE_C4,
  NOTE_C4
};

int durationsL[] = {
  4, 4, 8, 4, 8,
  4, 4, 8, 4, 8,
  4, 8, 4, 8, 4,
  1
};


// Nokia:
int melodyN[] = {
  NOTE_E5, NOTE_D5, NOTE_FS4, NOTE_GS4, 
  NOTE_CS5, NOTE_B4, NOTE_D4, NOTE_E4, 
  NOTE_B4, NOTE_A4, NOTE_CS4, NOTE_E4,
  NOTE_A4
};

int durationsN[] = {
  8, 8, 4, 4,
  8, 8, 4, 4,
  8, 8, 4, 4,
  2
};

// Pink Panther:
int melodyP[] = {
  REST, REST, REST, NOTE_DS4, 
  NOTE_E4, REST, NOTE_FS4, NOTE_G4, REST, NOTE_DS4,
  NOTE_E4, NOTE_FS4,  NOTE_G4, NOTE_C5, NOTE_B4, NOTE_E4, NOTE_G4, NOTE_B4,   
  NOTE_AS4, NOTE_A4, NOTE_G4, NOTE_E4, NOTE_D4, 
  NOTE_E4, REST, REST, NOTE_DS4
};

int durationsP[] = {
  2, 4, 8, 8, 
  4, 8, 8, 4, 8, 8,
  8, 8,  8, 8, 8, 8, 8, 8,   
  2, 16, 16, 16, 16, 
  2, 4, 8, 4
};




//=============================PARAMETERS==================================

// Getting the LDC:
LiquidCrystal_I2C lcd(0x27, 16, 2);

//Servo parameters:
Servo lockServo;         // Servo for locking mechanism
int lockPos = 15;        // Locked position limit
int unlockPos = 180;     // Unlocked position limit

// The length of the password (the actual length is 4):
#define Password_Length 5
int Position = 0; // position of the servo
// The password stored in a string:
char Particular[Password_Length]; 
// The password which is specifically set at the beginning:
char Specific[Password_Length] = "1234";
// Counts the amount of digits and and checks to see if the password is correct:
byte Particular_Count = 0, Specific_Count = 0;
char Key;
// The amount of rows and columns on the keypad:
const byte ROWS = 4;
const byte COLS = 4;
// Sets the keypad digits:
char keys[ROWS][COLS] = {
  {'1','2','3','A'}, 

  {'4','5','6','B'},

  {'7','8','9','C'},

  {'*','0','#','D'}
};

// Variable to check the state of the door (opened or closed):
bool SmartDoor = true;

// The pins where the keypad is plugged into:
byte rowPins[ROWS] = {7, 6, 5, 4};
byte colPins[COLS] = {3, 2, 1, 0};

// This function is used to assign the Arduino pins to the buttons
// (basically gets the data from the keypad):
Keypad myKeypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS);

//=============================SETUP==================================

void setup()
{
  // Attaches the servo to pin A0:
  lockServo.attach(A0);
  pinMode(BUZZER, OUTPUT); // Set buzzer - pin 9 as an output
  // Closing the safe (always at the beginning):
  ServoClose();
  // Initialising the LCD:
  lcd.init();
  // Turning on the backlight:
  lcd.backlight();
  // Setting the cursor on the first row and first caracter on LCD:
  lcd.setCursor(0,0);
  // Introduction messages:
  lcd.print("*");
  lcd.print("*");
  lcd.print("*");
  lcd.print("P");
  lcd.print("M");
  lcd.print(" ");
  lcd.print("P");
  lcd.print("r");
  lcd.print("o");
  lcd.print("j");
  lcd.print("e");
  lcd.print("c");
  lcd.print("t");
  lcd.print("*");
  lcd.print("*");
  lcd.print("*");
  // Setting the cursor on the second row and first caracter on LCD:
  lcd.setCursor(0,1);
  lcd.print("*");
  lcd.print("*");
  lcd.print("S");
  lcd.print("e");
  lcd.print("i");
  lcd.print("f");
  lcd.print(" ");
  lcd.print("D");
  lcd.print("e");
  lcd.print("s");
  lcd.print("t");
  lcd.print("e");
  lcd.print("p");
  lcd.print("t");
  lcd.print("*");
  lcd.print("*");
  nokia();
  delay(2000);
  // Clearing the LCD display:
  lcd.clear();

}

//=============================LOOP==================================

void loop()
{
  // If the door is open:  
  if (SmartDoor == 0)
  {
    // Getting the value on the keypad:
    Key = myKeypad.getKey();

    // If the '#' key is pressed, then the door will close:
    if (Key == '#')
    {
      // Clearing the LCD display:
      lcd.clear();
      // Closing the servo motor:
      ServoClose();
      // The closing message is going to be shown on the LCD:
      lcd.setCursor(0,0);
      lcd.print("*");
      lcd.print("*");
      lcd.print("D");
      lcd.print("o");
      lcd.print("o");
      lcd.print("r");
      lcd.print(" ");
      lcd.print("C");
      lcd.print("l");
      lcd.print("o");
      lcd.print("s");
      lcd.print("e");
      lcd.print("d");
      lcd.print("!");
      lcd.print("*");
      lcd.print("*");
      delay(3000); // waits 3 seconds
      // The variable is set for closed door:
      SmartDoor = 1;
    }
  }

  // Else keeping the door open:
  else Open();
}

//=============================FUNCTIONS==================================

// Function which clears the data entered on the keypad:
void clearData()
{
   // Counting the pressed digits:
  while (Particular_Count != 0)
  {
    // Erasing the old values:
    Particular[Particular_Count--] = 0;
  }
  return;
}


// Function which opens the door:
void ServoOpen()
{
  lockServo.write(unlockPos);
}


// Function which closes the door:
void ServoClose()
{
  lockServo.write(lockPos);
}


// Function which verifies the password:
void Open()
{
  // The program is waiting for a password:
  lcd.setCursor(0,0);
  lcd.print("E");
  lcd.print("n");
  lcd.print("t");
  lcd.print("e");
  lcd.print("r");
  lcd.print(" ");
  lcd.print("P");
  lcd.print("a");
  lcd.print("s");
  lcd.print("s");
  lcd.print("w");
  lcd.print("o");
  lcd.print("r");
  lcd.print("d");
  lcd.print(":");

  // Getting the keys pressed on the keypad:
  Key = myKeypad.getKey();

  // If a key was pressed:
  if (Key)
  {
    // The key is stored on Particular_Count position in the string
    // that later will be compared the the specific password:
    Particular[Particular_Count] = Key;
    // After each key pressed, a "*" character is going to be shown on the LCD:
    lcd.setCursor(Particular_Count, 1);
    lcd.print("*");
    // Counting the length of the password introduced:
    Particular_Count++;
  }

  // If the password introduced has the same length as the correct one, then the
  // two strings will be compared and the door will open or stay close accordingly
  // to the result:
  if (Particular_Count == Password_Length - 1)
  {
    if (!strcmp(Particular, Specific))
    {
      // Opening the door:
      lionSleepsTonight();
      lcd.clear();
      ServoOpen();
      lcd.setCursor(0,0);
      lcd.print("D");
      lcd.print("o");
      lcd.print("o");
      lcd.print("r");
      lcd.print(" ");
      lcd.print("O");
      lcd.print("p");
      lcd.print("e");
      lcd.print("n");
      lcd.print("e");
      lcd.print("d");
      lcd.print("!");
      lcd.setCursor(0,0);
      
      lcd.setCursor(0,1);
      lcd.print("P");
      lcd.print("r");
      lcd.print("e");
      lcd.print("s");
      lcd.print("s");
      lcd.print(" ");
      lcd.print("#");
      lcd.print(" ");
      lcd.print("t");
      lcd.print("o");
      lcd.print(" ");
      lcd.print("C");
      lcd.print("l");
      lcd.print("o");
      lcd.print("s");
      lcd.print("e");
      lcd.print("!");
      SmartDoor = 0;
    }
    else // The password is wrong, keep the door closed:
    {
      pinkPanther();
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("W");
      lcd.print("r");
      lcd.print("o");
      lcd.print("n");
      lcd.print("g");
      lcd.print(" ");
      lcd.print("P");
      lcd.print("a");
      lcd.print("s");
      lcd.print("s");
      lcd.print("w");
      lcd.print("o");
      lcd.print("r");
      lcd.print("d");
      lcd.print("!");
      
      delay(2000);

      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("T");
      lcd.print("r");
      lcd.print("y");
      lcd.print(" ");
      lcd.print("A");
      lcd.print("g");
      lcd.print("a");
      lcd.print("i");
      lcd.print("n");
      lcd.print("!");
      
      delay(2000);
      SmartDoor = 1;
    }
    clearData();
  }
}


// The functions for the songs:
void lionSleepsTonight () {
  int size = sizeof(durationsL) / sizeof(int);

  for (int note = 0; note < size; note++) {
    //to calculate the note duration, take one second divided by the note type.
    //e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
    int duration = 1000 / durationsL[note];
    tone(BUZZER, melodyL[note], duration);

    //to distinguish the notes, set a minimum time between them.
    //the note's duration + 30% seems to work well:
    int pauseBetweenNotes = duration * 1.30;
    delay(pauseBetweenNotes);
    
    //stop the tone playing:
    noTone(BUZZER);
  }
}

void nokia () {
  int size = sizeof(durationsN) / sizeof(int);

  for (int note = 0; note < size; note++) {
    //to calculate the note duration, take one second divided by the note type.
    //e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
    int duration = 1000 / durationsN[note];
    tone(BUZZER, melodyN[note], duration);

    //to distinguish the notes, set a minimum time between them.
    //the note's duration + 30% seems to work well:
    int pauseBetweenNotes = duration * 1.30;
    delay(pauseBetweenNotes);
    
    //stop the tone playing:
    noTone(BUZZER);
  }
}


void pinkPanther () {
  int size = sizeof(durationsP) / sizeof(int);

  for (int note = 0; note < size; note++) {
    //to calculate the note duration, take one second divided by the note type.
    //e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
    int duration = 1000 / durationsP[note];
    tone(BUZZER, melodyP[note], duration);

    //to distinguish the notes, set a minimum time between them.
    //the note's duration + 30% seems to work well:
    int pauseBetweenNotes = duration * 1.30;
    delay(pauseBetweenNotes);
    
    //stop the tone playing:
    noTone(BUZZER);
  }
}
